==> vala-0.3.2.tar.bz2: vala-0.3.2/vala/valaelementaccess.c <==
/* valaelementaccess.vala
 *
 * Copyright (C) 2006-2008  Raffaele Sandrini, Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Raffaele Sandrini <raffaele@sandrini.ch>
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <vala/valaelementaccess.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include <gee/readonlylist.h>
#include <vala/valasourcereference.h>
#include <vala/valacodevisitor.h>
#include <vala/valacodegenerator.h>




struct _ValaElementAccessPrivate {
	ValaExpression* _container;
	GeeList* indices;
};

#define VALA_ELEMENT_ACCESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_ELEMENT_ACCESS, ValaElementAccessPrivate))
enum  {
	VALA_ELEMENT_ACCESS_DUMMY_PROPERTY,
	VALA_ELEMENT_ACCESS_CONTAINER
};
static void vala_element_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_element_access_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node);
static gboolean vala_element_access_real_is_pure (ValaExpression* base);
static ValaCodeBinding* vala_element_access_real_create_code_binding (ValaCodeNode* base, ValaCodeGenerator* codegen);
static gpointer vala_element_access_parent_class = NULL;
static void vala_element_access_dispose (GObject * obj);



void vala_element_access_append_index (ValaElementAccess* self, ValaExpression* index) {
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (self));
	g_return_if_fail (VALA_IS_EXPRESSION (index));
	gee_collection_add (GEE_COLLECTION (self->priv->indices), index);
	vala_code_node_set_parent_node (VALA_CODE_NODE (index), VALA_CODE_NODE (self));
}


GeeList* vala_element_access_get_indices (ValaElementAccess* self) {
	g_return_val_if_fail (VALA_IS_ELEMENT_ACCESS (self), NULL);
	return GEE_LIST (gee_read_only_list_new (VALA_TYPE_EXPRESSION, ((GBoxedCopyFunc) g_object_ref), g_object_unref, self->priv->indices));
}


ValaElementAccess* vala_element_access_new (ValaExpression* container, ValaSourceReference* source_reference) {
	ValaElementAccess * self;
	g_return_val_if_fail (VALA_IS_EXPRESSION (container), NULL);
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
	self = g_object_newv (VALA_TYPE_ELEMENT_ACCESS, 0, NULL);
	vala_code_node_set_source_reference (VALA_CODE_NODE (self), source_reference);
	vala_element_access_set_container (self, container);
	return self;
}


static void vala_element_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (self->priv->_container), visitor);
	{
		GeeList* e_collection;
		GeeIterator* e_it;
		e_collection = self->priv->indices;
		e_it = gee_iterable_iterator (GEE_ITERABLE (e_collection));
		while (gee_iterator_next (e_it)) {
			ValaExpression* e;
			e = ((ValaExpression*) gee_iterator_get (e_it));
			{
				vala_code_node_accept (VALA_CODE_NODE (e), visitor);
				(e == NULL ? NULL : (e = (g_object_unref (e), NULL)));
			}
		}
		(e_it == NULL ? NULL : (e_it = (g_object_unref (e_it), NULL)));
	}
	vala_code_visitor_visit_element_access (visitor, self);
}


static void vala_element_access_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node) {
	ValaElementAccess * self;
	gint index;
	self = VALA_ELEMENT_ACCESS (base);
	g_return_if_fail (VALA_IS_EXPRESSION (old_node));
	g_return_if_fail (VALA_IS_EXPRESSION (new_node));
	if (self->priv->_container == old_node) {
		vala_element_access_set_container (self, new_node);
	}
	index = gee_list_index_of (self->priv->indices, old_node);
	if (index >= 0 && vala_code_node_get_parent_node (VALA_CODE_NODE (new_node)) == NULL) {
		gee_list_set (((GeeList*) self->priv->indices), index, new_node);
		vala_code_node_set_parent_node (VALA_CODE_NODE (new_node), VALA_CODE_NODE (self));
	}
}


static gboolean vala_element_access_real_is_pure (ValaExpression* base) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (base);
	{
		GeeList* index_collection;
		GeeIterator* index_it;
		index_collection = self->priv->indices;
		index_it = gee_iterable_iterator (GEE_ITERABLE (index_collection));
		while (gee_iterator_next (index_it)) {
			ValaExpression* index;
			index = ((ValaExpression*) gee_iterator_get (index_it));
			{
				if (!vala_expression_is_pure (index)) {
					gboolean _tmp0;
					return (_tmp0 = FALSE, (index == NULL ? NULL : (index = (g_object_unref (index), NULL))), (index_it == NULL ? NULL : (index_it = (g_object_unref (index_it), NULL))), _tmp0);
				}
				(index == NULL ? NULL : (index = (g_object_unref (index), NULL)));
			}
		}
		(index_it == NULL ? NULL : (index_it = (g_object_unref (index_it), NULL)));
	}
	return vala_expression_is_pure (self->priv->_container);
}


static ValaCodeBinding* vala_element_access_real_create_code_binding (ValaCodeNode* base, ValaCodeGenerator* codegen) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (base);
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (codegen), NULL);
	return vala_code_generator_create_element_access_binding (codegen, self);
}


ValaExpression* vala_element_access_get_container (ValaElementAccess* self) {
	g_return_val_if_fail (VALA_IS_ELEMENT_ACCESS (self), NULL);
	return self->priv->_container;
}


void vala_element_access_set_container (ValaElementAccess* self, ValaExpression* value) {
	ValaExpression* _tmp2;
	ValaExpression* _tmp1;
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_container = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_container == NULL ? NULL : (self->priv->_container = (g_object_unref (self->priv->_container), NULL))), _tmp2);
}


static void vala_element_access_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (object);
	switch (property_id) {
		case VALA_ELEMENT_ACCESS_CONTAINER:
		g_value_set_object (value, vala_element_access_get_container (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_element_access_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (object);
	switch (property_id) {
		case VALA_ELEMENT_ACCESS_CONTAINER:
		vala_element_access_set_container (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_element_access_class_init (ValaElementAccessClass * klass) {
	vala_element_access_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaElementAccessPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_element_access_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_element_access_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_element_access_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_element_access_real_accept;
	VALA_CODE_NODE_CLASS (klass)->replace_expression = vala_element_access_real_replace_expression;
	VALA_EXPRESSION_CLASS (klass)->is_pure = vala_element_access_real_is_pure;
	VALA_CODE_NODE_CLASS (klass)->create_code_binding = vala_element_access_real_create_code_binding;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ELEMENT_ACCESS_CONTAINER, g_param_spec_object ("container", "container", "container", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_element_access_init (ValaElementAccess * self) {
	self->priv = VALA_ELEMENT_ACCESS_GET_PRIVATE (self);
	self->priv->indices = GEE_LIST (gee_array_list_new (VALA_TYPE_EXPRESSION, ((GBoxedCopyFunc) g_object_ref), g_object_unref, g_direct_equal));
}


static void vala_element_access_dispose (GObject * obj) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (obj);
	(self->priv->_container == NULL ? NULL : (self->priv->_container = (g_object_unref (self->priv->_container), NULL)));
	(self->priv->indices == NULL ? NULL : (self->priv->indices = (g_object_unref (self->priv->indices), NULL)));
	G_OBJECT_CLASS (vala_element_access_parent_class)->dispose (obj);
}


GType vala_element_access_get_type (void) {
	static GType vala_element_access_type_id = 0;
	if (G_UNLIKELY (vala_element_access_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaElementAccessClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_element_access_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaElementAccess), 0, (GInstanceInitFunc) vala_element_access_init };
		vala_element_access_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaElementAccess", &g_define_type_info, 0);
	}
	return vala_element_access_type_id;
}

==> vala-0.3.5.tar.bz2: vala-0.3.5/vala/valaelementaccess.c <==
/* valaelementaccess.vala
 *
 * Copyright (C) 2006-2008  Raffaele Sandrini, Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Raffaele Sandrini <raffaele@sandrini.ch>
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <vala/valaelementaccess.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include <gee/readonlylist.h>
#include <vala/valasourcereference.h>
#include <vala/valacodevisitor.h>
#include <vala/valacodegenerator.h>




struct _ValaElementAccessPrivate {
	ValaExpression* _container;
	GeeList* indices;
};

#define VALA_ELEMENT_ACCESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_ELEMENT_ACCESS, ValaElementAccessPrivate))
enum  {
	VALA_ELEMENT_ACCESS_DUMMY_PROPERTY,
	VALA_ELEMENT_ACCESS_CONTAINER
};
static void vala_element_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_element_access_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_element_access_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node);
static gboolean vala_element_access_real_is_pure (ValaExpression* base);
static ValaCodeBinding* vala_element_access_real_create_code_binding (ValaCodeNode* base, ValaCodeGenerator* codegen);
static gpointer vala_element_access_parent_class = NULL;
static void vala_element_access_finalize (GObject * obj);



void vala_element_access_append_index (ValaElementAccess* self, ValaExpression* index) {
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (self));
	g_return_if_fail (VALA_IS_EXPRESSION (index));
	gee_collection_add (GEE_COLLECTION (self->priv->indices), index);
	vala_code_node_set_parent_node (VALA_CODE_NODE (index), VALA_CODE_NODE (self));
}


GeeList* vala_element_access_get_indices (ValaElementAccess* self) {
	g_return_val_if_fail (VALA_IS_ELEMENT_ACCESS (self), NULL);
	return GEE_LIST (gee_read_only_list_new (VALA_TYPE_EXPRESSION, ((GBoxedCopyFunc) (g_object_ref)), g_object_unref, self->priv->indices));
}


ValaElementAccess* vala_element_access_new (ValaExpression* container, ValaSourceReference* source_reference) {
	ValaElementAccess * self;
	g_return_val_if_fail (VALA_IS_EXPRESSION (container), NULL);
	g_return_val_if_fail (VALA_IS_SOURCE_REFERENCE (source_reference), NULL);
	self = g_object_newv (VALA_TYPE_ELEMENT_ACCESS, 0, NULL);
	vala_code_node_set_source_reference (VALA_CODE_NODE (self), source_reference);
	vala_element_access_set_container (self, container);
	return self;
}


static void vala_element_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_element_access (visitor, self);
	vala_code_visitor_visit_expression (visitor, VALA_EXPRESSION (self));
}


static void vala_element_access_real_accept_children (ValaCodeNode* base, ValaCodeVisitor* visitor) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (self->priv->_container), visitor);
	{
		GeeList* e_collection;
		int e_it;
		e_collection = self->priv->indices;
		for (e_it = 0; e_it < gee_collection_get_size (GEE_COLLECTION (e_collection)); e_it = e_it + 1) {
			ValaExpression* e;
			e = ((ValaExpression*) (gee_list_get (GEE_LIST (e_collection), e_it)));
			{
				vala_code_node_accept (VALA_CODE_NODE (e), visitor);
				(e == NULL ? NULL : (e = (g_object_unref (e), NULL)));
			}
		}
	}
}


static void vala_element_access_real_replace_expression (ValaCodeNode* base, ValaExpression* old_node, ValaExpression* new_node) {
	ValaElementAccess * self;
	gint index;
	self = VALA_ELEMENT_ACCESS (base);
	g_return_if_fail (VALA_IS_EXPRESSION (old_node));
	g_return_if_fail (VALA_IS_EXPRESSION (new_node));
	if (self->priv->_container == old_node) {
		vala_element_access_set_container (self, new_node);
	}
	index = gee_list_index_of (self->priv->indices, old_node);
	if (index >= 0 && vala_code_node_get_parent_node (VALA_CODE_NODE (new_node)) == NULL) {
		gee_list_set (((GeeList*) (self->priv->indices)), index, new_node);
		vala_code_node_set_parent_node (VALA_CODE_NODE (new_node), VALA_CODE_NODE (self));
	}
}


static gboolean vala_element_access_real_is_pure (ValaExpression* base) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (base);
	{
		GeeList* index_collection;
		int index_it;
		index_collection = self->priv->indices;
		for (index_it = 0; index_it < gee_collection_get_size (GEE_COLLECTION (index_collection)); index_it = index_it + 1) {
			ValaExpression* index;
			index = ((ValaExpression*) (gee_list_get (GEE_LIST (index_collection), index_it)));
			{
				if (!vala_expression_is_pure (index)) {
					gboolean _tmp0;
					return (_tmp0 = FALSE, (index == NULL ? NULL : (index = (g_object_unref (index), NULL))), _tmp0);
				}
				(index == NULL ? NULL : (index = (g_object_unref (index), NULL)));
			}
		}
	}
	return vala_expression_is_pure (self->priv->_container);
}


static ValaCodeBinding* vala_element_access_real_create_code_binding (ValaCodeNode* base, ValaCodeGenerator* codegen) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (base);
	g_return_val_if_fail (VALA_IS_CODE_GENERATOR (codegen), NULL);
	return vala_code_generator_create_element_access_binding (codegen, self);
}


ValaExpression* vala_element_access_get_container (ValaElementAccess* self) {
	g_return_val_if_fail (VALA_IS_ELEMENT_ACCESS (self), NULL);
	return self->priv->_container;
}


void vala_element_access_set_container (ValaElementAccess* self, ValaExpression* value) {
	ValaExpression* _tmp2;
	ValaExpression* _tmp1;
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_container = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_container == NULL ? NULL : (self->priv->_container = (g_object_unref (self->priv->_container), NULL))), _tmp2);
	g_object_notify (((GObject *) (self)), "container");
}


static void vala_element_access_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (object);
	switch (property_id) {
		case VALA_ELEMENT_ACCESS_CONTAINER:
		g_value_set_object (value, vala_element_access_get_container (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_element_access_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (object);
	switch (property_id) {
		case VALA_ELEMENT_ACCESS_CONTAINER:
		vala_element_access_set_container (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vala_element_access_class_init (ValaElementAccessClass * klass) {
	vala_element_access_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaElementAccessPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_element_access_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_element_access_set_property;
	G_OBJECT_CLASS (klass)->finalize = vala_element_access_finalize;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_element_access_real_accept;
	VALA_CODE_NODE_CLASS (klass)->accept_children = vala_element_access_real_accept_children;
	VALA_CODE_NODE_CLASS (klass)->replace_expression = vala_element_access_real_replace_expression;
	VALA_EXPRESSION_CLASS (klass)->is_pure = vala_element_access_real_is_pure;
	VALA_CODE_NODE_CLASS (klass)->create_code_binding = vala_element_access_real_create_code_binding;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ELEMENT_ACCESS_CONTAINER, g_param_spec_object ("container", "container", "container", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_element_access_instance_init (ValaElementAccess * self) {
	self->priv = VALA_ELEMENT_ACCESS_GET_PRIVATE (self);
	self->priv->indices = GEE_LIST (gee_array_list_new (VALA_TYPE_EXPRESSION, ((GBoxedCopyFunc) (g_object_ref)), g_object_unref, g_direct_equal));
}


static void vala_element_access_finalize (GObject * obj) {
	ValaElementAccess * self;
	self = VALA_ELEMENT_ACCESS (obj);
	(self->priv->_container == NULL ? NULL : (self->priv->_container = (g_object_unref (self->priv->_container), NULL)));
	(self->priv->indices == NULL ? NULL : (self->priv->indices = (g_object_unref (self->priv->indices), NULL)));
	G_OBJECT_CLASS (vala_element_access_parent_class)->finalize (obj);
}


GType vala_element_access_get_type (void) {
	static GType vala_element_access_type_id = 0;
	if (vala_element_access_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaElementAccessClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_element_access_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaElementAccess), 0, (GInstanceInitFunc) vala_element_access_instance_init };
		vala_element_access_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaElementAccess", &g_define_type_info, 0);
	}
	return vala_element_access_type_id;
}
